﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Win32;
using Path = System.IO.Path;

namespace MellotProgrammer
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window, IDataOutput
    {
        private Connector _connector;
        private string _lastPath;

        public MainWindow()
        {
            InitializeComponent();
            SetVersionToTitle();
            SetupConnector();
        }

        private void SetVersionToTitle()
        {
            Title += " (" + GetVersion() + ")";
        }

        private string GetVersion()
        {
            var version = Assembly.GetExecutingAssembly().GetName().Version;
            return version.Major + "." + version.Minor + "." + version.Build + "." + version.Revision;
        }

        public void SetupConnector()
        {
            _connector = new Connector(this);
            SetupComPorts();
            SetSelectedComPort();
        }

        private void SetSelectedComPort()
        {
            if (ComboBoxPorts.Items.Count > 0)
            {
                ComboBoxPorts.SelectedIndex = 0;
            }
        }

        private void SetupComPorts()
        {
            var comPortNames = _connector.GetComPortNames();
            foreach (var comPortName in comPortNames)
            {
                ComboBoxPorts.Items.Add(comPortName);
            }
        }

        public void OutputData(string data)
        {
            Application.Current.Dispatcher.BeginInvoke(new Action(() =>
            {
                TextBoxReadData.AppendText(data);
                TextBoxReadData.ScrollToEnd();
            }));
        }

        private void ButtonConnect_OnClick(object sender, RoutedEventArgs e)
        {
            if (ComboBoxPorts.Items.Count > 0)
            {
                _connector.Connect(ComboBoxPorts.SelectedItem as string);
                _connector.PrintVersion();
                SetEnableButtonStates();
                ShowOutputBox();
            }
        }

        private void ShowOutputBox()
        {
            ImageStartup.Visibility = Visibility.Collapsed;
            TextBoxReadData.Visibility = Visibility.Visible;
        }

        private void SetEnableButtonStates()
        {
            if (_connector.IsConnected())
            {
                ButtonConnect.IsEnabled = false;
                ButtonDump.IsEnabled = true;
                ButtonDumpRam.IsEnabled = true;
                ButtonFlash.IsEnabled = true;
                ButtonSend.IsEnabled = true;
                ButtonEraseFlash.IsEnabled = true;
                ButtonEraseRam.IsEnabled = true;
                ButtonPrintMenu.IsEnabled = true;
                ButtonRestart.IsEnabled = true;
                ButtonDumpToFile.IsEnabled = true;
                ButtonDumpRamToFile.IsEnabled = true;
            }
        }

        private void ButtonDump_OnClick(object sender, RoutedEventArgs e)
        {
            ClearReadData();
            _connector.Dump();
        }

        private void ClearReadData()
        {
            TextBoxReadData.Text = "";
        }

        private void ButtonDumpRam_OnClick(object sender, RoutedEventArgs e)
        {
            ClearReadData();
            _connector.DumpRam();
        }

        private void ButtonFlash_OnClick(object sender, RoutedEventArgs e)
        {
            ClearReadData();
            _connector.Flash();
        }

        private void ButtonSend_OnClick(object sender, RoutedEventArgs e)
        {
            if (TextBoxData.Text.Any())
            {
                ClearReadData();
                _connector.SendData(TextBoxData.Text);
            }
        }

        private void EraseFlash_OnClick(object sender, RoutedEventArgs e)
        {
            ClearReadData();
            _connector.EraseFlash();
        }

        private void EraseRam_OnClick(object sender, RoutedEventArgs e)
        {
            ClearReadData();
            _connector.EraseRam();
        }

        private void ButtonPrintMenu_OnClick(object sender, RoutedEventArgs e)
        {
            ClearReadData();
            _connector.PrintMenu();
        }

        private void ButtonRestart_OnClick(object sender, RoutedEventArgs e)
        {
            ClearReadData();
            _connector.Restart();
        }

        private void ButtonDumpToFile_OnClick(object sender, RoutedEventArgs e)
        {
            var saveFilePath = GetSaveFilePath("Virtual Boy Files (*.vb)|*.vb");
            if (!string.IsNullOrEmpty(saveFilePath))
            {
                ClearReadData();
                _connector.DumpRomToFile(saveFilePath);
            }
        }

        private void ButtonDumpRamToFile_OnClick(object sender, RoutedEventArgs e)
        {
            var saveFilePath = GetSaveFilePath("Virtual Boy RAM Files (*.sram)|*.sram");
            if (!string.IsNullOrEmpty(saveFilePath))
            {
                ClearReadData();
                _connector.DumpRamToFile(saveFilePath);
            }
        }

        private string GetSaveFilePath(string filter)
        {
            SaveFileDialog dialog = new SaveFileDialog
            {
                Filter = filter,
                OverwritePrompt = true,
                InitialDirectory = _lastPath ?? AppDomain.CurrentDomain.BaseDirectory
            };

            return GetPathFromDialog(dialog);
        }

        private string GetPathFromDialog(SaveFileDialog dialog)
        {
            var result = dialog.ShowDialog();

            if (result == true)
            {
                _lastPath = Path.GetDirectoryName(dialog.FileName);
                return dialog.FileName;
            }

            return null;
        }
    }
}
